"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = _default;

var _chalk = _interopRequireDefault(require("chalk"));

var _path = require("path");

var _lodash = _interopRequireDefault(require("lodash.flatten"));

var _codeFrame = _interopRequireDefault(require("@babel/code-frame"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* eslint-disable no-console */
function _default(babel) {
  var t = babel.types,
      transform = babel.transform; // Cache the bad code's loc, when exit exportDefaultDeclaration
  // inject `console.warn(...)` after `export default` statement.
  // Could not inject `console.warn(...)` just at the place where
  // the bad code located, because the bad code may would not be
  // executed after module loaded.

  var cache = null;

  function getWarnMessage(loc) {
    return "Probably missing \"yield\" keyword at method calling. (".concat(loc.line, ":").concat(loc.column, ")");
  }

  function getCalleeMeta(node) {
    var callee = node.callee;
    var name;

    if (t.isIdentifier(callee)) {
      name = callee.name;
    } else if (t.isMemberExpression(callee)) {
      var obj = callee.object;

      if (t.isCallExpression(obj)) {
        name = getCalleeMeta(obj).name;
      } else {
        while (obj && !obj.name) {
          obj = obj.object;
        }

        if (obj) {
          name = obj.name;
        }
      }
    }

    return {
      name: name,
      loc: callee.loc.start
    };
  }

  function getNameFromObjectPattern(node) {
    return t.isObjectPattern(node) ? node.properties.map(function (prop) {
      if (t.isObjectPattern(prop.value)) {
        return getNameFromObjectPattern(prop.value);
      } else if (t.Identifier(prop.value)) {
        return prop.value.name;
      }

      return null;
    }) : null;
  }

  function getEffectAndActionNames(params) {
    var names = params.map(function (param) {
      if (t.isIdentifier(param)) {
        return param.name;
      }

      return getNameFromObjectPattern(param);
    });
    return (0, _lodash.default)(names, true);
  }

  function visitCallExpression(paramNames, path) {
    if (!t.isYieldExpression(path.parent)) {
      var _getCalleeMeta = getCalleeMeta(path.node),
          name = _getCalleeMeta.name,
          loc = _getCalleeMeta.loc; // is effect or action call


      if (paramNames.includes(name)) {
        var filename = path.hub.file.opts.filename;
        var filepath = "./".concat((0, _path.relative)(process.cwd(), filename));
        console.log();
        console.log(_chalk.default.inverse(filepath));
        console.log(getWarnMessage(loc));
        var code = (0, _codeFrame.default)(path.hub.file.code, loc.line, loc.column, {
          highlightCode: true
        });
        console.log();
        console.log(code);
        console.log();
        console.log();

        if (process.env.NODE_ENV !== 'production' && cache) {
          cache.push({
            file: filepath,
            loc: loc
          });
        }
      }
    }
  }

  function visitMethod(path) {
    var node = path.node;
    var params;
    var generator = false;

    if (t.isObjectMethod(node)) {
      generator = node.generator;
      params = node.params;
    } else if (t.isObjectProperty(node)) {
      var val = node.value;

      if (t.isFunctionExpression(val) && val.generator) {
        generator = true;
        params = val.params;
      }
    }

    if (generator) {
      var paramNames = getEffectAndActionNames(params.slice(1));
      path.get('body').traverse({
        CallExpression: visitCallExpression.bind(null, paramNames)
      });
    }
  }

  var visitor = {
    'ObjectMethod|ObjectProperty': visitMethod
  };

  if (process.env.NODE_ENV !== 'production') {
    visitor.ExportDefaultDeclaration = {
      enter: function enter() {
        cache = [];
      },
      exit: function exit(path) {
        if (cache && cache.length) {
          cache.forEach(function (_ref) {
            var file = _ref.file,
                loc = _ref.loc;
            var code = (0, _codeFrame.default)(path.hub.file.code, loc.line, loc.column);
            var raw = "console.warn('".concat(file, " \\n").concat(getWarnMessage(loc), " \\n\\n").concat(code.split('\n').join('\\n'), "\\n ')");
            var ret = transform(raw, {
              code: true,
              babelrc: false
            });
            path.insertAfter(ret.ast.program.body[0]);
          });
        }
      }
    };
  }

  return {
    visitor: visitor
  };
}

module.exports = exports["default"];